@@ -435,7 +435,7 @@ class TestIntegrationCluster < TestIntegration
     cli_server "-C test/config/prune_bundler_with_multiple_workers.rb"
     reply = read_body(connect)
 
-    assert reply, "embedded app"
+    assert_equal reply, "embedded app"
   end
 
   def test_load_path_includes_extra_deps
@@ -575,8 +575,21 @@ class TestIntegrationCluster < TestIntegration
       wait_for_server_to_match(/(index \d data \d)/, 1)
     end.sort
 
-    assert 'index 0 data 0', ary[0]
-    assert 'index 1 data 1', ary[1]
+    assert_equal 'index 0 data 0', ary[0]
+    assert_equal 'index 1 data 1', ary[1]
+  end
+
+  def test_after_worker_shutdown_hook
+    cli_server "-C test/config/after_shutdown_hook.rb test/rackup/hello.ru"
+    get_worker_pids 0, 2 # make sure workers are booted
+    stop_server
+
+    ary = Array.new(2) do |_index|
+      wait_for_server_to_match(/(after_worker_shutdown worker=\d status=\d+)/, 1)
+    end.sort
+
+    assert_equal 'after_worker_shutdown worker=0 status=0', ary[0]
+    assert_equal 'after_worker_shutdown worker=1 status=0', ary[1]
   end
 
   def test_worker_hook_warning_cli
