@@ -53,7 +53,17 @@ class TestThreadPool < PumaTest
     # If +wait+ is true, wait until the trim request is completed before returning.
     def trim(force=false, wait: true)
       super(force)
-      Thread.pass until @trim_requested == 0 if wait
+
+      return unless wait
+      loop do
+        # While we wait until @trim_requested is 0, the test might inspect other values
+        # such as @spawned which may be modified after this variable is modified in the loop
+        # Lock for race safety
+        with_mutex do
+          return if @trim_requested == 0
+        end
+        Thread.pass
+      end
     end
   end
 
@@ -174,7 +184,7 @@ class TestThreadPool < PumaTest
 
     assert_equal 1, pool.spawned
 
-    pool.trim
+    pool.trim(wait: true)
     assert_equal 0, pool.spawned
   end
 
@@ -185,10 +195,10 @@ class TestThreadPool < PumaTest
 
     assert_equal 2, pool.spawned
 
-    pool.trim
+    pool.trim(wait: true)
     assert_equal 1, pool.spawned
 
-    pool.trim
+    pool.trim(wait: true)
     assert_equal 1, pool.spawned
   end
 
@@ -340,7 +350,8 @@ class TestThreadPool < PumaTest
     timeout = 0.01
     grace = 0.01
 
-    rescued = []
+    # JRuby Array#<< is not thread-safe
+    rescued = Queue.new
     pool = mutex_pool(2, 2, pool_shutdown_grace_time: grace) do
       begin
         pool.with_force_shutdown do
@@ -360,7 +371,10 @@ class TestThreadPool < PumaTest
 
     assert_equal 0, pool.spawned
     assert_equal 2, rescued.length
-    refute rescued.compact.any?(&:alive?)
+    until rescued.empty?
+      thread = rescued.pop
+      refute thread.alive?
+    end
   end
 
   def test_correct_waiting_count_for_killed_threads
